# -*- coding: utf-8 -*-
#
# Copyright (c) 2021, HILSTER GmbH, Germany - https://hilster.io
# All rights reserved.
#

from htf.communication.core import SSHClientBase


class SSHClient(SSHClientBase):

    def __init__(self):
        """
        ``SSHClient`` executes commands remotely using ssh.
        """
        SSHClientBase.__init__(self)

    def connect(self, hostname, username=None, password=None):
        """
        Establish an ssh connection.

        Args:
            hostname (str): the hostname or ip address
            username=None (str): The username to be used. If set to ``None`` the current username is used instead
             of using the ssh keys in your system.
            password=None (str): The password for ``username``. If set to ``None`` the password for ``username``
             or the current username is used.
        """

        self._client.connect(hostname, username=username, password=password)
        self._connected = True

    def _check_connection(self):
        if self._connected is False:
            raise Exception("Not connected. Call connect() first!")

    def close(self):
        """
        Close current ssh connection.
        """
        self._client.close()
        self._connected = False

    def execute(self, *command):
        """
        Execute a command remotely via ssh.

        Args:
            *command (tuple of str): the command to be executed.

        Returns:
            tuple (exitcode, stdout, stderr): a tuple containing exitcode, stdout and stderr of the executed command.
        """
        self._check_connection()

        if len(command) > 1:
            cmd = " ".join(command)
        else:
            cmd = command[0]
        # print("executing command:", cmd)

        stdin, stdout, stderr = self._client.exec_command(cmd)

        return stdout.channel.recv_exit_status(), stdout.read(), stderr.read()


class SFTPClient(SSHClient):

    def __init__(self):
        """
        ``SFTPClient`` lets you access files via sftp protocol.
        """
        super(SFTPClient, self).__init__()
        self._sftpclient = None

    def connect(self, hostname, username=None, password=None):
        """
        Establish an sftp connection.

        Args:
            hostname (str): the hostname or ip address
            username=None (str): The username to be used. If set to ``None`` the current username is used instead
             of using the ssh keys in your system.
            password=None (str): The password for ``username``. If set to ``None`` the password for ``username``
             or the current username is used.
        """

        self._client.connect(hostname, username=username, password=password)
        self._sftpclient = self._client.open_sftp()

    def _check_connection(self):
        if self._sftpclient is None:
            raise Exception("Not connected. Call connect() first!")

    def close(self):
        """
        Close current sftp connection.
        """
        self._check_connection()
        self._client.close()
        self._sftpclient = None

    def get(self, remotepath, localpath):
        """
        Copies file identified by ``remotepath`` to ``localpath`` using sftp protocol.

        Args:
            remotepath (str): the remote file to be copied
            localpath (str): the local path where the field is copied to
        """
        self._check_connection()

        return self._sftpclient.get(remotepath, localpath)

    def put(self, localpath, remotepath):
        """
        Copies file identified by ``localpath`` to ``remotepath`` using sftp protocol.

        Args:
            localpath (str): the local path to be copied
            remotepath (str): the remote file where the field is copied to

        Returns:
            SFTPAttributes: the result of os.stat

        """
        self._check_connection()

        return self._sftpclient.put(localpath, remotepath)

    def remove(self, remotepath):
        """
        Removes a file identified by ``remotepath``.

        Args:
            remotepath (str): the path for the file to be removed

        Raises:
            IOError: if the path is not a file
        """
        self._check_connection()

        return self._sftpclient.remove(remotepath)

    unlink = remove

    def rename(self, oldpath, newpath):
        """
        Remotely renames a file or a folder.

        Args:
            oldpath (str): the file or folder to be renamed
            newpath (str): the new file- or foldername

        Raises:
            IOError: if something goes wrong
        """
        self._check_connection()

        return self._sftpclient.rename(oldpath, newpath)
